package com.imcode.forum.sql;

import com.imcode.forum.ForumPost;
import com.imcode.forum.Id;
import com.imcode.forum.SimpleForumPost;

import java.util.*;

class SqlPostsSet extends AbstractSet<ForumPost> {

    private SqlPost newestPost ;
    private SqlPost topPost ;
    private Map<Id, SortedSet<ForumPost>> parentPostIdToRepliesMap = new HashMap<Id, SortedSet<ForumPost>>();
    private Map<Id, ForumPost> posts = new HashMap<Id, ForumPost>();

    public boolean add(ForumPost fp) {
        SqlPost post = (SqlPost) fp ;
        if (null == newestPost || post.getDateTime().after(newestPost.getDateTime())) {
            newestPost = post ;
        }
        Id parentPostId = post.getParentPostId();
        if (null == parentPostId ) {
            topPost = post ;
        } else {
            SortedSet<ForumPost> repliesSet = getRepliesToPostWithId(parentPostId);
            repliesSet.add(post) ;
        }

        return null == posts.put(post.getId(), post) ;
    }

    private SortedSet<ForumPost> getRepliesToPostWithId(Id parentPostId) {
        SortedSet<ForumPost> repliesSet = parentPostIdToRepliesMap.get(parentPostId);
        if (null == repliesSet) {
            repliesSet = new TreeSet<ForumPost>(SimpleForumPost.DATETIME_COMPARATOR);
            parentPostIdToRepliesMap.put(parentPostId, repliesSet) ;
        }
        return repliesSet;
    }

    ForumPost getNewestPost() {
        return newestPost;
    }

    ForumPost getTopPost() {
        return topPost;
    }

    SortedSet<ForumPost> getPostReplies(Id postId) {
        return getRepliesToPostWithId(postId) ;
    }

    ForumPost getPost(Id postId) {
        return posts.get(postId);
    }

    public Iterator<ForumPost> iterator() {
        return posts.values().iterator() ;
    }

    public int size() {
        return posts.size() ;
    }

}
