package com.imcode.forum.sql;

import com.imcode.forum.ForumPost;
import com.imcode.forum.ForumThread;
import com.imcode.forum.Id;

import java.util.SortedSet;

class SqlThread implements ForumThread {

    private final Id id;
    private SqlRepository sqlRepository;
    private SqlPostsSet postsSet;

    SqlThread(Id id,
              SqlRepository sqlRepository) {
        this.id = id;
        this.sqlRepository = sqlRepository;
    }

    public ForumPost getTopPost() {
        return getSqlPosts().getTopPost() ;
    }

    public ForumPost getNewestPost() {
        return getSqlPosts().getNewestPost() ;
    }

    public Id getId() {
        return id;
    }

    private SqlPostsSet getSqlPosts() {
        if (null == postsSet ) {
            postsSet = sqlRepository.getThreadPosts(this);
        }
        return postsSet ;
    }

    public ForumPost getPost(Id postId) {
        return getSqlPosts().getPost(postId) ;
    }

    public int getPostCount() {
        return getSqlPosts().size();
    }

    public String toString() {
        return "thread " + id + " with "+getSqlPosts().size() +" posts, top " + getTopPost() + ", and newest "
               + getNewestPost();
    }

    Id createPostReply(Id postId, ForumPost forumPost) {
        Id replyId = sqlRepository.createPostReply(id, postId, forumPost);
        postsSet = null ;
        return replyId ;
    }

    SortedSet<ForumPost> getPostReplies(final ForumPost post) {
        return getSqlPosts().getPostReplies(post.getId()) ;
    }

    void incrementPostViewCount(Id postId) {
        sqlRepository.incrementPostViewCount(postId);
    }

    void hidePost(Id postId, boolean hidden) {
        sqlRepository.hidePost(id, postId, hidden) ;
    }
}
